/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
/*
 * pkix_pl_mem.h
 *
 * Memory Management Definitions
 *
 */

#ifndef _PKIX_PL_MEM_H
#define _PKIX_PL_MEM_H

#include "pkix_pl_common.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

#endif /* _PKIX_PL_MEM_H */
